import { Notice, Plugin, requestUrl } from "obsidian";
import {
	BookTrackerPluginSettings,
	BookTrackerSettingTab,
	DEFAULT_SETTINGS,
} from "./settings/settings";
import { getBookByLegacyId } from "@data-sources/goodreads/scraper";
import { Templater } from "./utils/templater";
import { GoodreadsSearchModal } from "@views/goodreads-search-modal";
import { GoodreadsSearchSuggestModal } from "@views/goodreads-search-suggest-modal";

const CONTENT_TYPE_EXTENSIONS: Record<string, string> = {
	"image/jpeg": "jpg",
	"image/png": "png",
	"image/gif": "gif",
	"image/webp": "webp",
};

export default class BookTrackerPlugin extends Plugin {
	settings: BookTrackerPluginSettings;
	templater: Templater;

	async onload() {
		await this.loadSettings();

		this.templater = new Templater(this.app);

		this.addCommand({
			id: "search-goodreads",
			name: "Search Goodreads",
			callback: () => this.searchGoodreads(),
		});

		this.addSettingTab(new BookTrackerSettingTab(this.app, this));
	}

	onunload() {}

	async loadSettings() {
		this.settings = Object.assign(
			{},
			DEFAULT_SETTINGS,
			await this.loadData()
		);
	}

	async saveSettings() {
		await this.saveData(this.settings);
	}

	async downloadCoverImage(
		coverImageUrl: string,
		fileName: string
	): Promise<string> {
		const response = await requestUrl(coverImageUrl);
		const contentType = response.headers["content-type"];
		const extension = CONTENT_TYPE_EXTENSIONS[contentType || ""] || "";
		if (extension === "") {
			throw new Error("Unsupported content type: " + contentType);
		}

		let filePath = this.settings.coverDirectory + "/";
		if (this.settings.groupCoversByFirstLetter) {
			let groupName = fileName.charAt(0).toUpperCase();
			if (!/^[A-Z]$/.test(groupName)) {
				groupName = "#";
			}

			filePath += groupName + "/";
		}
		filePath += fileName + "." + extension;

		const existingFile = this.app.vault.getFileByPath(filePath);
		if (existingFile) {
			if (this.settings.overwriteExistingCovers) {
				await this.app.vault.modifyBinary(
					existingFile,
					response.arrayBuffer
				);
			} else {
				new Notice("Cover image already exists: " + filePath);
				return filePath;
			}
		}

		await this.app.vault.createBinary(filePath, response.arrayBuffer);

		return filePath;
	}

	async createEntryFromGoodreads(legacyId: number): Promise<void> {
		try {
			const book = await getBookByLegacyId(legacyId);

			const fileName = this.templater
				.renderTemplate(this.settings.fileNameFormat, {
					title: book.title,
					authors: book.authors.map((a) => a.name).join(", "),
				})
				.replace(/[/\:*?<>|""]/g, "");

			const data: Record<string, unknown> = { book };

			if (this.settings.downloadCovers && book.coverImageUrl) {
				data.coverImagePath = await this.downloadCoverImage(
					book.coverImageUrl,
					fileName
				);
			}

			const renderedContent = await this.templater.renderTemplateFile(
				this.settings.templateFile,
				data
			);

			if (renderedContent) {
				await this.app.vault.create(
					this.settings.tbrDirectory + "/" + fileName + ".md",
					renderedContent
				);
			}
		} catch (error) {
			console.error("Failed to create book entry:", error);
		}
	}

	async searchGoodreads(): Promise<void> {
		const results = await GoodreadsSearchModal.createAndOpen(this.app);
		const selectedBook = await GoodreadsSearchSuggestModal.createAndOpen(
			this.app,
			results
		);

		if (selectedBook) {
			await this.createEntryFromGoodreads(selectedBook.legacyId);
		} else {
			new Notice("No book selected.");
		}
	}
}
